// spi.h
#ifndef SPI_H
#define SPI_H

#include "../main.h"

#define SPI_MOSI	PB5
#define SPI_MISO	PB6
#define SPI_SCK		PB7
#define SPI_SS		PB4
#define SPI_DDR		DDRB
#define SPI_PORT	PORTB 

typedef enum
{
  MOD_0 = 1<<SPE | 1<<MSTR,
  MOD_1 = 1<<SPE | 1<<MSTR | 1<<CPHA,
  MOD_2 = 1<<SPE | 1<<MSTR | 1<<CPOL,
  MOD_3 = 1<<SPE | 1<<MSTR | 1<<CPOL | 1<<CPHA,
} spimod_t;

typedef enum
{
  DIV_4,
  DIV_2,
  DIV_16,
  DIV_8,
  DIV_64,
  DIV_32,
  DIV_128,
} spidiv_t;

typedef enum
{
  MSB_FIRST,
  LSB_FIRST  = 1<<DORD,
} spiord_t;

typedef enum
{
  SMOD_0 = 1<<SPE,
  SMOD_1 = 1<<SPE | 1<<CPHA,
  SMOD_2 = 1<<SPE | 1<<CPOL,
  SMOD_3 = 1<<SPE | 1<<CPOL | 1<<CPHA,
} spislavemod_t;

static __inline__ u8 spi_master_transmit(u8 data) __attribute__((__always_inline__));
u8 spi_master_transmit(u8 data)
{
	/* Start transmission */
	SPDR = data;
	/* Wait for transmission complete */
	while(!(SPSR & (1<<SPIF)));
	return SPDR;
}

static __inline__ u8 spi_slave_receive(void) __attribute__((__always_inline__));

u8 spi_slave_receive(void){
	/* Wait for reception complete */
	while(!(SPSR & (1<<SPIF)));
	/* Return Data Register */
	return SPDR;
}

static __inline__ void spi_portmasterinit(void) __attribute__((__always_inline__));
void spi_portmasterinit(void){
	SPI_PORT |= (1<<SPI_SS)|(1<<SPI_MOSI)|(1<<SPI_SCK)|(1<<SPI_MISO);
	SPI_DDR |= (1<<SPI_SS)|(1<<SPI_MOSI)|(1<<SPI_SCK);
}

static __inline__ void spi_portslaveinit(void) __attribute__((__always_inline__));
void spi_portslaveinit(void){
	SPI_PORT |= (1<<SPI_SS)|(1<<SPI_MOSI)|(1<<SPI_SCK)|(1<<SPI_MISO);
	SPI_DDR &= ~(1<<SPI_SS)|(1<<SPI_MOSI)|(1<<SPI_SCK);
	SPI_DDR |= (1<<SPI_MISO);
}

// mode = MOD_0, MOD_1, MOD_2, MOD_3
// order = MSB_FIRST, LSB_FIRST
// geschw = DIV_2,DIV_4,DIV_8,DIV_16,DIV_32,DIV_64,DIV_128 
static __inline__ void spi_master_setmode(spidiv_t geschw, spimod_t mode, spiord_t order) __attribute__((__always_inline__));
static inline void spi_master_setmode(spidiv_t geschw, spimod_t mode, spiord_t order)
{
  SPCR = mode | order | geschw>>1;
  SPSR = geschw & 1;
} 


static __inline__ void spi_init(spidiv_t geschw, spimod_t mode, spiord_t order) __attribute__((__always_inline__));
void spi_init(spidiv_t geschw, spimod_t mode, spiord_t order){
	spi_portmasterinit();
	spi_master_setmode(geschw, mode, order);
	spi_master_transmit(0); 
}

// SPI als Slave
// mode = MOD_0, MOD_1, MOD_2, MOD_3
// order = MSB_FIRST, LSB_FIRST
static __inline__ void spi_slave_init(spislavemod_t mode, spiord_t order) __attribute__((__always_inline__));
static inline void spi_slave_init(spislavemod_t mode, spiord_t order)
{
	spi_portslaveinit();
  	SPCR = mode | order;
}

/* SPI intern */

/* Adressen von internen Geraeten */
#define SD_ADR 		0
#define ZIF_ADR 	1
#define LED_ADR 	2
#define GF_ADR 		3
#define LCD_ADR 	4
#define KNOPF_ADR	4
#define BAR_ADR 	5
#define RAM_ADR 	6
#define UHR_ADR 	7 

#define SPI_INTERN_E_PORT 	PORTC
#define SPI_INTERN_E_DDR	DDRC
#define SPI_INTERN_E		PC6

#define SPI_INTERN_A0_PORT	PORTA
#define SPI_INTERN_A0_DDR	DDRA
#define SPI_INTERN_A0		PA6
#define SPI_INTERN_A1_PORT	PORTA
#define SPI_INTERN_A1_DDR	DDRA
#define SPI_INTERN_A1		PA7
#define SPI_INTERN_A2_PORT	PORTC
#define SPI_INTERN_A2_DDR	DDRC
#define SPI_INTERN_A2		PC7

static __inline__ void spi_intern_init(void) __attribute__((__always_inline__));
static inline void spi_intern_init(void){
	SPI_INTERN_E_PORT |= (1<<SPI_INTERN_E);
	SPI_INTERN_A0_PORT &= ~(1<<SPI_INTERN_A0);
	SPI_INTERN_A1_PORT &= ~(1<<SPI_INTERN_A1);
	SPI_INTERN_A2_PORT &= ~(1<<SPI_INTERN_A2);
	SPI_INTERN_E_DDR |= (1<<SPI_INTERN_E);
	SPI_INTERN_A0_DDR |= (1<<SPI_INTERN_A0);
	SPI_INTERN_A1_DDR |= (1<<SPI_INTERN_A1);
	SPI_INTERN_A2_DDR |= (1<<SPI_INTERN_A2);
}


static __inline__ void spi_intern_start(void) __attribute__((__always_inline__));
static inline void spi_intern_start(void){
	SPI_INTERN_E_PORT &= ~(1<<SPI_INTERN_E);
}

static __inline__ void spi_intern_stop(void) __attribute__((__always_inline__));
static inline void spi_intern_stop(void){
	SPI_INTERN_E_PORT |= (1<<SPI_INTERN_E);
} 

// CS-Adresse fuer interne SPI
static __inline__ void spi_intern_outadr(u8 adr) __attribute__((__always_inline__));
static inline void spi_intern_outadr(u8 adr){

	if(adr & (1<<0)){
		SPI_INTERN_A0_PORT |= (1<<SPI_INTERN_A0);
	}else{
		SPI_INTERN_A0_PORT &= ~(1<<SPI_INTERN_A0);
	}
	if(adr & (1<<1)){
		SPI_INTERN_A1_PORT |= (1<<SPI_INTERN_A1);
	}else{
		SPI_INTERN_A1_PORT &= ~(1<<SPI_INTERN_A1);
	}
	if(adr & (1<<2)){
		SPI_INTERN_A2_PORT |= (1<<SPI_INTERN_A2);
	}else{
		SPI_INTERN_A2_PORT &= ~(1<<SPI_INTERN_A2);
	}
}


#endif	/* SPI_H */
